﻿using Microsoft.Extensions.Logging;
using System.Windows.Forms;
using static System.Windows.Forms.DataFormats;

namespace PIbd_22_Kalyshev_Y_V_MotorBoat_Base
{
    public partial class FormMapWithSetBoats : Form
    {
        /// <summary>
        /// Словарь для выпадающего списка
        /// </summary>
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
        {
            { "Простая карта", new SimpleMap() },
            { "Собственная карта", new MySecondMap() }
        };
        /// <summary>
        /// Объект от коллекции карт
        /// </summary>
        private readonly MapsCollection _mapsCollection;
        /// <summary>
        /// Логер
        /// </summary>
        private readonly ILogger _logger;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormMapWithSetBoats(ILogger<FormMapWithSetBoats> logger)
        {
            InitializeComponent();
            _logger = logger;
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }
        /// <summary>
        /// Заполнение listBoxMaps
        /// </summary>
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex; listBoxMaps.Items.Clear();
            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }
            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }
        /// <summary>
        /// Добавление карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("Нет такой карты", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text, _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
            _logger.LogInformation($"Добавлена карта {textBoxNewMapName.Text}");
        }
        /// <summary>
        /// Выбор карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ListBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }
        /// <summary>
        /// Удаление карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить карту {listBoxMaps.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
            }
            _logger.LogInformation($"Удалена карта {listBoxMaps.SelectedItem}");
        }
        /// <summary>
        /// <summary>
        /// Добавление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {
            var formBoatConfig = new FormBoatConfig();
            formBoatConfig.AddEvent(AddBoat);
            formBoatConfig.Show();
            _logger.LogInformation($"Удалена карта {listBoxMaps.SelectedItem}");
        }
        private void AddBoat(DrawningBoat drawingBoats)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            DrawningObjectBoat boat = new DrawningObjectBoat(drawingBoats);
            try
            {
                if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] + boat != -1)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    _logger.LogInformation($"В карту {_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty]} добавлен объект {boat.GetInfo()}");
                }
                else
                {
                    MessageBox.Show("Невозможно добавить объект");
                    _logger.LogInformation($"Невозможно добавить объект {boat.GetInfo()} в карту {_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty]}");
                }
            } catch (StorageOverflowException ex)
            {
                MessageBox.Show($"Ошибка добавления: {ex.Message}");
                _logger.LogWarning($"Ошибка добавления: {ex.Message}");
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Неизвестная ошибка: {ex.Message}");
                _logger.LogWarning($"Неизвестная ошибка: {ex.Message}");
            }

        }
        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            try
            {
                IDrawningObject boat = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] - pos;
                if (boat != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
                    _logger.LogInformation($"Из карты {_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty]} удален объект {boat.GetInfo()}");
                }
                else
                {
                    MessageBox.Show("Не удалось удалить объект");
                    _logger.LogInformation($"Невозможно удалить объект {boat.GetInfo()} в карту {_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty]}");
                }
            } catch (BoatNotFoundException ex)
            {
                MessageBox.Show($"Ошибка удаления: {ex.Message}");
                _logger.LogWarning($"Ошибка удаления: {ex.Message}");
            } catch (Exception ex)
            {
                MessageBox.Show($"Неизвестная ошибка: {ex.Message}");
                _logger.LogWarning($"Неизвестная ошибка: {ex.Message}");
            }
        }
        /// <summary>
        /// Вывод набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            _logger.LogInformation($"Показана гавань для карты {_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty]}");
        }
        /// <summary>
        /// Вывод карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowOnMap();
            _logger.LogInformation($"Показана карта {_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty]}");
        }
        /// <summary>
        /// Перемещение
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            //получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "buttonUp":
                    dir = Direction.Up;
                    break;
                case "buttonDown":
                    dir = Direction.Down;
                    break;
                case "buttonLeft":
                    dir = Direction.Left;
                    break;
                case "buttonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ??
            string.Empty].MoveObject(dir);
        }
        /// <summary>
        /// Обработка нажатия "Сохранение"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Сохранен файл {saveFileDialog.FileName}");
                } catch(Exception ex)
                {
                    MessageBox.Show($"Не сохранилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Не сохранилось: {ex.Message}");
                }
            }
        }
        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _mapsCollection.LoadData(openFileDialog.FileName);
                    ReloadMaps();
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Загружен файл {openFileDialog.FileName}");
                } catch (Exception ex)
                {
                    MessageBox.Show("Не загрузилось", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Не загрузилось: {ex.Message}");
                }
            }
        }
    }
}